#include "serialbus.h"
#include "debug.h"

#ifdef DRIVER_DEBUG
ULONG DebugMask = 0xFFFFFFFF;

const CHAR IrpNames[][IRP_NAMES_MAX_LEN] = {
    "IRP_MJ_CREATE",
    "IRP_MJ_CREATE_NAMED_PIPE",
    "IRP_MJ_CLOSE",
    "IRP_MJ_READ",
    "IRP_MJ_WRITE",
    "IRP_MJ_QUERY_INFORMATION",
    "IRP_MJ_SET_INFORMATION",
    "IRP_MJ_QUERY_EA",
    "IRP_MJ_SET_EA",
    "IRP_MJ_FLUSH_BUFFERS",
    "IRP_MJ_QUERY_VOLUME_INFORMATION",
    "IRP_MJ_SET_VOLUME_INFORMATION",
    "IRP_MJ_DIRECTORY_CONTROL",
    "IRP_MJ_FILE_SYSTEM_CONTROL",
    "IRP_MJ_DEVICE_CONTROL",
    "IRP_MJ_INTERNAL_DEVICE_CONTROL",
    "IRP_MJ_SHUTDOWN",
    "IRP_MJ_LOCK_CONTROL",
    "IRP_MJ_CLEANUP",
    "IRP_MJ_CREATE_MAILSLOT",
    "IRP_MJ_QUERY_SECURITY",
    "IRP_MJ_SET_SECURITY",
    "IRP_MJ_POWER",
    "IRP_MJ_SYSTEM_CONTROL",
    "IRP_MJ_DEVICE_CHANGE",
    "IRP_MJ_QUERY_QUOTA",
    "IRP_MJ_SET_QUOTA",
    "IRP_MJ_PNP"
};



const CHAR PnpMinorFuncNames[][PNP_MINOR_FUNC_NAMES_MAX_LEN] = {
	"IRP_MN_START_DEVICE",                    // 0x00
	"IRP_MN_QUERY_REMOVE_DEVICE",             // 0x01
	"IRP_MN_REMOVE_DEVICE",                   // 0x02
	"IRP_MN_CANCEL_REMOVE_DEVICE",            // 0x03
	"IRP_MN_STOP_DEVICE",                     // 0x04
	"IRP_MN_QUERY_STOP_DEVICE",               // 0x05
	"IRP_MN_CANCEL_STOP_DEVICE",              // 0x06

	"IRP_MN_QUERY_DEVICE_RELATIONS",          // 0x07
	"IRP_MN_QUERY_INTERFACE",                 // 0x08
	"IRP_MN_QUERY_CAPABILITIES",              // 0x09
	"IRP_MN_QUERY_RESOURCES",                 // 0x0A
	"IRP_MN_QUERY_RESOURCE_REQUIREMENTS",     // 0x0B
	"IRP_MN_QUERY_DEVICE_TEXT",               // 0x0C
	"IRP_MN_FILTER_RESOURCE_REQUIREMENTS",    // 0x0D
  "",                                       // 0x0E

	"IRP_MN_READ_CONFIG",                     // 0x0F
	"IRP_MN_WRITE_CONFIG",                    // 0x10
	"IRP_MN_EJECT",                           // 0x11
	"IRP_MN_SET_LOCK",                        // 0x12
	"IRP_MN_QUERY_ID",                        // 0x13
	"IRP_MN_QUERY_PNP_DEVICE_STATE",          // 0x14
	"IRP_MN_QUERY_BUS_INFORMATION",           // 0x15
	"IRP_MN_DEVICE_USAGE_NOTIFICATION",       // 0x16
	"IRP_MN_SURPRISE_REMOVAL",                // 0x17
	"IRP_MN_QUERY_LEGACY_BUS_INFORMATION"     // 0x18
};


#endif // DRIVER_DEBUG

VOID
DriverDbgSetDebugMask(ULONG NewDebugMask)
{
#ifdef DRIVER_DEBUG
  DriverDbg( ("New DebugMask - 0x%08X\n", NewDebugMask) );
  DebugMask = NewDebugMask;
#else // !DRIVER_DEBUG
  UNREFERENCED_PARAMETER(NewDebugMask);
#endif // DRIVER_DEBUG
}


VOID
DriverDbgDumpData(PCHAR Buffer, ULONG BufferSize)
{
#ifdef DRIVER_DEBUG
  ULONG i,j=0;

  DbgPrint("\n[%s] Data Dump\n    ", DRIVER_NAME);
  for(i=0; i<BufferSize; i++, j++)
  {
    if( 16 == j)
    {
      DbgPrint("\n    ");
      j=0;
    }
    DbgPrint("%02X ", Buffer[i]);
  }
  DbgPrint("\n\n    ");
  
  for(i=0, j=0; i<BufferSize; i++, j++)
  {
    DbgPrint("%c", Buffer[i]);
  }
  DbgPrint("\n\n");

#else // !DERIVER_DEBUG
  UNREFERENCED_PARAMETER(Buffer);
  UNREFERENCED_PARAMETER(BufferSize);
#endif // DRIVER_DEBUG
}

VOID
DriverDbgTest(VOID)
{
#ifdef DRIVER_DEBUG
  ULONG PrevDebugMask = DebugMask;

  DriverDbgSetDebugMask(DBGMSKMAIN | DBGLVLERROR);
  DbgPrint("\nDriverDbgTest:\n");
  DbgPrint("  [0x%08X]  DBGMASKMAIN | DBGLVLERROR - begin\n", DebugMask);
  DriverDbgPrint( DBGLVLERROR, DBGMSKMAIN,  ("    [OK] - DBGLVLERROR, DBGMSKMAIN\n") );
  DriverDbgPrint( DBGLVLERROR, DBGMSKPNP,   ("    [FAIL] - DBGLVLERROR, DBGMSKPNP\n") );
  DriverDbgPrint( DBGLVLERROR, DBGMSKIOCTL, ("    [FAIL] - DBGLVLERROR, DBGMSKMAIN\n") );
  DriverDbgPrint( DBGLVLWARN, DBGMSKMAIN,   ("    [FAIL] - DBGLVLWARN, DBGMSKMAIN\n") );
  DbgPrint("  DBGMASKMAIN | DBGLVLERROR - end\n\n");

  DriverDbgSetDebugMask(DBGLVLTRACE | DBGMSKPOWER);
  DbgPrint("  [0x%08X]  DBGLVLTRACE | DBGMSKPOWER - begin\n", DebugMask);
  DriverDbgPrint( DBGLVLERROR, DBGMSKPOWER,   ("    [OK] - DBGLVLERROR, DBGMSKPOWER\n") );
  DriverDbgPrint( DBGLVLWARN, DBGMSKPOWER,    ("    [OK] - DBGLVLWARN, DBGMSKPOWER\n") );
  DriverDbgPrint( DBGLVLINFO, DBGMSKPOWER,    ("    [OK] - DBGLVLINFO, DBGMSKPOWER\n") );
  DriverDbgPrint( DBGLVLTRACE, DBGMSKPOWER,   ("    [OK] - DBGLVLTRACE, DBGMSKPOWER\n") );
  DriverDbgPrint( DBGLVLERROR, DBGMSKMAIN,    ("    [FAIL] - DBGLVLERROR, DBGMSKMAIN\n") );
  DbgPrint("  DBGLVLTRACE, DBGMSKPOWER - end\n\n");

  DriverDbgSetDebugMask(PrevDebugMask);
#endif // DRIVER_DEBUG
}